/*
 * Copyright (c) 2008-2013 Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX,Ext,Sonatype,Nexus*/
/**
 * This ldap grid will list the ldap servers currently configured, and contains
 * a toolbar with many functions, including adding new items, deleting items,
 * changing order of items, and saving that order.
 */
NX.define('Sonatype.repoServer.LdapGrid', {
  extend : 'Ext.grid.GridPanel',

  requires: [
    'Nexus.ext.GridFilterBox'
  ],

  constructor : function(cfg) {
    var config = cfg || {};
    var defaultConfig = {};
    Ext.apply(this, config, defaultConfig);

    // this is the store that retrieves the ldap server basic config for display
    this.store = new Ext.data.JsonStore({
      root : 'data',
      id : 'url',
      fields : [
        {
          name : 'id',
          sortType : Ext.data.SortTypes.asUCString
        },
        {
          name : 'name',
          sortType : Ext.data.SortTypes.asUCString
        },
        {
          name : 'url',
          sortType : Ext.data.SortTypes.asUCString
        },
        {
          name : 'ldapUrl',
          sortType : Ext.data.SortTypes.asUCString
        }
      ],
      url : Sonatype.config.servicePath + '/ldap/servers',
      autoLoad : true,
      sortInfo : {
        field : 'name',
        direction : 'asc'
      },
      listeners : {
        beforeload : this.rememberSelection,
        load : this.recallSelection,
        scope : this
      }
    });


    // this constructor is used to create new grid items when you click
    // the add button, the grid items from server are created automatically
    // by the above json store
    this.recordConstructor = Ext.data.Record.create([
      {
        name : 'id'
      },
      {
        name : 'name'
      },
      {
        name : 'url'
      },
      {
        name : 'ldapUrl'
      }
    ]);

    // the column that will autofill the width of the grid
    this.autoExpandColumn = 'urlColumn';

    // very simple, only showing name and ldap url columns
    this.columns = [
      {
        header : 'Name',
        dataIndex : 'name',
        width : 200,
        sortable : true
      },
      {
        id : 'urlColumn',
        header : 'URL',
        dataIndex : 'ldapUrl',
        width : 300,
        sortable : true
      }
    ];

    // the grid toolbar, where all the buttons go for tasks
    this.tbar = [
      {
        text : 'Refresh',
        icon : Sonatype.config.resourcePath + '/images/icons/arrow_refresh.png',
        cls : 'x-btn-text-icon',
        scope : this,
        handler : this.refreshHandler
      },
      {
        text : 'Add',
        icon : Sonatype.config.resourcePath + '/images/icons/add.png',
        cls : 'x-btn-text-icon',
        scope : this,
        handler : this.addHandler,
        disabled : !this.sp.checkPermission('security:ldapconfig', this.sp.CREATE)
      },
      {
        text : 'Delete',
        icon : Sonatype.config.resourcePath + '/images/icons/delete.png',
        cls : 'x-btn-text-icon',
        scope : this,
        handler : this.deleteHandler,
        disabled : !this.sp.checkPermission('security:ldapconfig', this.sp.DELETE)
      },
      ' ',
      ' ',
      ' ',
      '-',
      ' ',
      ' ',
      ' ',
      {
        text : 'Move Up',
        icon : Sonatype.config.resourcePath + '/static/images/move-up.gif',
        cls : 'x-btn-text-icon',
        tooltip : {
          title : 'LDAP Server Ordering',
          text : 'Move the LDAP Server up in the list priority.'
        },
        scope : this,
        handler : this.moveUpHandler,
        disabled : !this.sp.checkPermission('security:ldapconfig', this.sp.EDIT)
      },
      {
        text : 'Move Down',
        icon : Sonatype.config.resourcePath + '/static/images/move-down.gif',
        cls : 'x-btn-text-icon',
        tooltip : {
          title : 'LDAP Server Ordering',
          text : 'Move the LDAP Server down in the list priority.'
        },
        scope : this,
        handler : this.moveDownHandler,
        disabled : !this.sp.checkPermission('security:ldapconfig', this.sp.EDIT)
      },
      {
        text : 'Save Order',
        icon : Sonatype.config.extPath + '/resources/images/default/dd/drop-yes.gif',
        cls : 'x-btn-text-icon',
        tooltip : {
          title : 'LDAP Server Ordering',
          text : 'Save the current LDAP Server priority order.'
        },
        scope : this,
        handler : this.saveOrderHandler,
        disabled : !this.sp.checkPermission('security:ldapconfig', this.sp.EDIT)
      },
      ' ',
      ' ',
      ' ',
      '-',
      ' ',
      ' ',
      ' ',
      {
        text : 'Clear Cache',
        icon : Sonatype.config.resourcePath + '/images/icons/user-trash.png',
        cls : 'x-btn-text-icon',
        tooltip : {
          title : 'Clear LDAP Cache',
          text : 'Clear the current cache of LDAP authentication and authorization credentials.'
        },
        scope : this,
        handler : this.clearCacheHandler,
        disabled : !this.sp.checkPermission('security:ldapconfig', this.sp.DELETE)
      }
    ];

    Sonatype.repoServer.LdapGrid.superclass.constructor.call(this, {
      viewConfig: {
        emptyText: 'No LDAP servers',
        emptyTextWhileFiltering: 'No LDAP servers matched criteria: {criteria}'
      },
      sm : new Ext.grid.RowSelectionModel({
        singleSelect : true
      }),
      listeners : {
        rowclick : {
          fn : this.rowclickHandler,
          scope : this
        },
        afterrender : function(cmp) {
          var mask = new Ext.LoadMask(cmp.getGridEl(), {store : this.store});
        }
      }
    });

    this.getTopToolbar().add([
      '->',
      NX.create('Nexus.ext.GridFilterBox', {
        filteredGrid: this
      })
    ]);
  },

  // this is called by the ldap container when a save occurs and
  // we need to update data for a record. Note that we need to
  // first remove then change then read the record for changes
  // to properly get applied. Then of course sorting, and select
  // current record
  updateRecord : function(id, jsonData) {
    var rec = this.store.getById(id);

    this.store.remove(rec);

    rec.id = jsonData.id;
    rec.data.id = jsonData.id;
    rec.data.name = jsonData.name;
    rec.data.url = jsonData.url
    rec.data.ldapUrl =
          jsonData.connectionInfo.protocol + '://' + jsonData.connectionInfo.host + ':' + jsonData.connectionInfo.port
                + '/' + jsonData.connectionInfo.searchBase;

    this.store.add(rec);

    this.store.sort('name', 'ASC');

    this.getSelectionModel().selectRecords([rec], false);

    return rec;
  },
  // refresh the list in the grid from the server
  // TODO: need to notify the container of this, so items can
  // be removed from the card panel
  refreshHandler : function() {
    this.store.reload();
    this.ldapContainer.refresh();
  },
  // add a new ldap server, give it a unique id, which only lives until
  // the data is saved, add it to grid, select it, and notify container
  // that an item was added, so card can get created
  addHandler : function() {
    var id = 'new_' + new Date().getTime();
    var newRec = new this.recordConstructor({
      name : 'New LDAP',
      id : id,
      url : 'new',
      ldapUrl : 'Not Available'
    }, id);

    this.store.insert(0, [newRec]);
    this.getSelectionModel().selectRecords([newRec], false);

    if (this.ldapAddHandler) {
      this.ldapAddHandler.call(this.ldapContainer, this, newRec);
    }
  },
  // ask the user if they really want to delete, and send the delete message
  // if an existing record, otherwise just dump the record from grid
  deleteHandler : function() {
    var record = this.getSelectionModel().getSelected();

    if (record.get('id').indexOf('new_') == -1) {
      Sonatype.utils.defaultToNo();

      Sonatype.MessageBox.show({
        animEl : this.getEl(),
        title : 'Delete',
        msg : 'Delete LDAP Server "' + record.get('name') + '"?',
        buttons : Sonatype.MessageBox.YESNO,
        scope : this,
        icon : Sonatype.MessageBox.QUESTION,
        fn : function(btnName) {
          if (btnName == 'yes' || btnName == 'ok') {
            Ext.Ajax.request({
              url : record.get('url'),
              callback : this.deleteCallback,
              cbPassThru : {
                record : record
              },
              scope : this,
              method : 'DELETE'
            });
          }
        }
      });
    }
    else {
      this.deleteCallback({
        cbPassThru : {
          record : record
        }
      }, true);
    }
  },
  // remove the server from the grid, and notify the container
  // so that the proper card can be removed
  deleteCallback : function(options, isSuccess, response) {
    if (isSuccess) {
      var record = options.cbPassThru.record;
      this.getStore().remove(record);

      if (this.ldapDeleteHandler) {
        this.ldapDeleteHandler.call(this.ldapContainer, this, record);
      }
    }
    else {
      Sonatype.utils.connectionError(response, 'Error Deleting LDAP Configuration.');
    }
  },
  // user clicked on a row, simply notify the container so that
  // the proper card can be selected
  rowclickHandler : function(grid, rowindex, event) {
    if (this.ldapRowclickHandler) {
      this.ldapRowclickHandler.call(this.ldapContainer, this, this.getStore().getAt(rowindex));
    }
  },
  // user wants to reorder the list of ldap servers, so we move the
  // selected item up one level if space allows
  moveUpHandler : function() {
    if (this.getSelectionModel().hasSelection()) {
      var rec = this.getSelectionModel().getSelected();

      if (rec) {
        var i = this.store.indexOf(rec);

        if (i > 0) {
          this.store.remove(rec);
          this.store.insert(i - 1, rec);
          this.getSelectionModel().selectRecords([rec], false);
        }
      }
    }
  },
  // user wants to reorder the list of ldap servers, so we move the
  // selected item down one level if space allows
  moveDownHandler : function() {
    if (this.getSelectionModel().hasSelection()) {
      var rec = this.getSelectionModel().getSelected();

      if (rec) {
        var i = this.store.indexOf(rec);

        if (i < (this.store.getCount() - 1)) {
          this.store.remove(rec);
          this.store.insert(i + 1, rec);
          this.getSelectionModel().selectRecords([rec], false);
        }
      }
    }
  },
  // this will PUT request to the server, the order of ldap ids
  saveOrderHandler : function() {
    var idList = [];
    for (var i = 0; i < this.store.getCount(); i++) {
      var rec = this.store.getAt(i);

      if (rec.get('id').indexOf('new_') == -1) {
        idList[idList.length] = rec.get('id');
      }
    }

    Sonatype.utils.defaultToNo();
    Sonatype.MessageBox.show({
      animEl : this.getEl(),
      title : 'Save LDAP Server Order',
      msg : 'Save the current order of the LDAP Servers?',
      buttons : Sonatype.MessageBox.YESNO,
      scope : this,
      icon : Sonatype.MessageBox.QUESTION,
      fn : function(btnName) {
        if (btnName == 'yes' || btnName == 'ok') {
          Ext.Ajax.request({
            callback : function(options, isSuccess, response) {
              if (!isSuccess) {
                Sonatype.utils.connectionError(response, 'The server did not save the order of the LDAP Servers.');
              }
            },
            scope : this,
            method : 'PUT',
            url : Sonatype.config.servicePath + '/ldap/server_order',
            jsonData : {
              data : idList
            }
          });
        }
      }
    });
  },
  clearCacheHandler : function() {
    Sonatype.MessageBox.wait('Clearing LDAP cache...');
    Ext.Ajax.request({
      url : Sonatype.config.servicePath + '/ldap/clearcache',
      callback : function(options, isSuccess, response) {
        if (isSuccess) {
          Sonatype.MessageBox.alert('Clear Cache',
                'The LDAP cache has been cleared.').setIcon(Sonatype.MessageBox.INFO);
        }
        else {
          Sonatype.utils.connectionError(response, 'The server did not clear the LDAP cache.');
        }
      },
      scope : this,
      method : 'DELETE'
    });
  },
  /**
   * Remember what records are selected.
   *
   * @private
   */
  rememberSelection: function() {
    // ldap only supports single selection
    this.selectedRecord = this.getSelectionModel().getSelected();
  },

  /**
   * Recall selection from previous remembered selection.
   *
   * @private
   */
  recallSelection: function() {
    var self = this;

    if (self.selectedRecord === undefined ) {
      return;
    }

    self.ldapRowclickHandler.call(self.ldapContainer, self, self.selectedRecord);
  }
});